@@ -10,11 +10,11 @@
  * use this software as long as: there is no monetary profit gained
  * specifically from the use or reproduction of this software, it is not
  * sold, rented, traded or otherwise marketed, and this copyright notice is
- * included prominently in any copy made. 
+ * included prominently in any copy made.
  *
  * The author make no claims as to the fitness or correctness of this software
  * for any use whatsoever, and it is provided as is. Any use of this software
- * is at the user's own risk. 
+ * is at the user's own risk.
  *
  */
 
@@ -51,7 +51,12 @@ ZLEXTERN int mult;
 ZLEXTERN int insmode;
 
 /* cost of last update */
+#ifdef HAS_SELECT
 ZLEXTERN int cost;
+#endif
+
+/* number of lines displayed */
+ZLEXTERN int nlnct;
 
 /* flags associated with last command */
 ZLEXTERN int lastcmd;
@@ -65,8 +70,6 @@ ZLEXTERN int virangeflag;
 /* kludge to get cw and dw to work right */
 ZLEXTERN int wordflag;
 
-/* Another kludge to lazy cache the usernames. Win on large systems */
-ZLEXTERN int usernamescached;
 #endif
 
 /* last named command done */
@@ -75,14 +78,12 @@ ZLEXTERN int lastnamed;
 /* != 0 if we're done editing */
 ZLEXTERN int done;
 
-/* length of prompt on screen */
-ZLEXTERN int pptlen;
+/* length of prompt, rprompt on screen */
+ZLEXTERN int pptlen, ppt2len;
 
 /* current history line number */
 ZLEXTERN int histline;
 
-ZLEXTERN int eofsent;
-
 /* != 0 if we need to call resetvideo() */
 ZLEXTERN int resetneeded;
 
@@ -104,15 +105,18 @@ ZLEXTERN char *curhistline;
 /* the status line */
 ZLEXTERN char *statusline;
 
-/* 1 if a complete added a slash at the end of a directory name */
-ZLEXTERN int addedslash;
+/* !=0 if a complete added a suffix at the end of a completion */
+ZLEXTERN int addedsuffix;
+
+/* 1 if we expect special keys after completing a parameter name */
+ZLEXTERN int complexpect;
 
 /*
 	the current history line and cursor position for the top line
 	on the buffer stack
 */
 
-ZLEXTERN int stackhist,stackcs;
+ZLEXTERN int stackhist, stackcs;
 
 /* != 0 if we are in the middle of a menu completion */
 ZLEXTERN int menucmp;
@@ -120,8 +124,11 @@ ZLEXTERN int menucmp;
 /* != 0 if we are making undo records */
 ZLEXTERN int undoing;
 
+/* != 0 if executing a shell function called from zle */
+ZLEXTERN  int inzlefunc;
+
 /* last vi change buffer */
-ZLEXTERN int vichgbufsz,vichgbufptr,vichgflag;
+ZLEXTERN int vichgbufsz, vichgbufptr, vichgflag;
 ZLEXTERN char *vichgbuf;
 
 ZLEXTERN int viinsbegin;
@@ -130,31 +137,34 @@ typedef void bindfunc DCLPROTO((void));
 typedef bindfunc *F;
 
 struct key {
-	struct hashnode *next; int canfree; char *nam; /* hash data */
-	int func;			/* function code for this key */
-	char *str;			/* string corresponding to this key,
-								if func = z_sequenceleadin				 */
-	int len;				/* length of string */
-	};
+    struct hashnode *next;
+    char *nam;		/* hash data */
+    int func;		/* function code for this key */
+    char *str;		/* string corresponding to this key,
+			   if func = z_sendstring */ 
+    int len;		/* length of string */
+    int prefixct;	/* number of strings for which this is a prefix */
+};
+
 struct zlecmd {
-	char *name;			/* name of function */
-	F func;				/* handler function */
-	int flags;
-	};
+    char *name;			/* name of function */
+    F func;			/* handler function */
+    int flags;
+};
 
 /* undo event */
 
 struct undoent {
-	int pref;		/* number of initial chars unchanged */
-	int suff;		/* number of trailing chars unchanged */
-	int len;			/* length of changed chars */
-	int cs;			/* cursor pos before change */
-	char *change;	/* NOT null terminated */
-	};
+    int pref;			/* number of initial chars unchanged */
+    int suff;			/* number of trailing chars unchanged */
+    int len;			/* length of changed chars */
+    int cs;			/* cursor pos before change */
+    char *change;		/* NOT null terminated */
+};
 
 #define UNDOCT 64
 
-struct undoent undos[UNDOCT];
+ZLEXTERN struct undoent undos[UNDOCT];
 
 /* the line before last mod (for undo purposes) */
 ZLEXTERN unsigned char *lastline;
@@ -162,7 +172,7 @@ ZLEXTERN unsigned char *lastline;
 /* buffer specified with "x */
 ZLEXTERN int vibufspec;
 
-ZLEXTERN int undoct,lastcs;
+ZLEXTERN int undoct, lastcs;
 
 ZLEXTERN char *visrchstr;
 ZLEXTERN int visrchsense;
@@ -182,11 +192,9 @@ ZLEXTERN int visrchsense;
 
 typedef struct key *Key;
 
-ZLEXTERN int *bindtab;
-extern int emacsbind[256];
-ZLEXTERN int altbindtab[256],mainbindtab[256];
-extern int viinsbind[],vicmdbind[];
-ZLEXTERN int vimarkcs[27],vimarkline[27];
+ZLEXTERN int *bindtab, *mainbindtab;
+extern int emacsbind[], viinsbind[], vicmdbind[];
+ZLEXTERN int altbindtab[256];
 
 #define KRINGCT 8
 ZLEXTERN char *kring[KRINGCT];
@@ -342,7 +350,11 @@ ZLEXTERN char *vibuf[36];
 #define z_vibackwardword 146
 #define z_uplineorsearch 147
 #define z_downlineorsearch 148
-#define ZLECMDCOUNT 149
+#define z_pushinput 149
+#define z_pushpopinput 150
+#define z_historybeginningsearchbackward 151
+#define z_historybeginningsearchforward 152
+#define z_expandorcompleteprefix 153
+#define ZLECMDCOUNT 154
 
 extern struct zlecmd zlecmds[];
-
